#include <stdio.h>
#include <math.h>

/* Nota:
   Em alguns compiladores, para usar funções matemáticas é preciso, além do #include <math.h>
   acima, compilar o programa com a opção -lm ao final:
   
   gcc -o ex05 ex05.c -lm
*/

/* Função que retorna o tamanho (número de dígitos) de um número. */
long tamanho(long num) {
	int tam = 1, div = 10;
	while ((num / div) > 0) { tam++; div *= 10; }
	return tam;
}

/* Função que retorna o enésimo dígito (da direita para a esquerda) de um número. */
long digito(long num, long posicao) {
	// Para obter o dígito, obtemos o resto da divisão por 10^posição, depois dividimos por (10^(posição-1)).
	// Ex.: para o número 567:
	//	- Posição 1: 567 % 10 = 7, 7 / 1 = 7.
	//	- Posição 2: 567 % 100 = 67, 67 / 10 = 6.
	// 	- Posição 3: 567 % 1000 = 567, 567 / 100 = 5.
	long div = pow(10, posicao);
	return (num % div) / (div / 10);
}

/* Função que retorna um dígito verificador utilizando o esquema Módulo 11. */
long dvModulo11(long num) {
	long tam, pos, mult, dig, soma = 0;
	
	// Realiza a soma dos dígitos do número multiplicados por 2, 3, 4, etc.
	tam = tamanho(num);
	for (pos = 1, mult = 2; pos <= tam; pos++, mult++) {
		dig = digito(num, pos);
		soma += dig * mult;
	}
	
	// Retorna a diferença entre a base 11 e o resto da divisão da soma por 11.
	return 11 - (soma % 11);
}

/* Função principal. */
main() {
	long baseCPF, dv;
	
	// Lê a base do CPF.
	printf("Digite a base de um CPF (sem os DVs): ");
	scanf("%ld", &baseCPF);
	
	// Adiciona os DVs, um de cada vez. Caso o DV seja 10 ou 11, usa-se 0.
	dv = dvModulo11(baseCPF);
	if (dv > 9) dv = 0;
	baseCPF = baseCPF * 10 + dv;
	dv = dvModulo11(baseCPF);
	if (dv > 9) dv = 0;
	baseCPF = baseCPF * 10 + dv;
	
	// Imprime o número de CPF completo.
	printf("CPF completo: %ld\n", baseCPF);
}
